/***************************************************************************
 *  Copyright (C) 2009 Nippon Telegraph and Telephone Corporation
 ***************************************************************************/

BEGIN;

-- Adjust this setting to control where the objects get created.
CREATE SCHEMA targetdb;
SET search_path = targetdb;

-- Register functions and views. (pg_filesystem)
CREATE FUNCTION pg_fs_size(text)
RETURNS bigint
AS '$libdir/target_db'
LANGUAGE C STRICT;

CREATE FUNCTION pg_fs_avail(text)
RETURNS bigint
AS '$libdir/target_db'
LANGUAGE C STRICT;

CREATE FUNCTION pg_fs_devid(text)
RETURNS bigint
AS '$libdir/target_db'
LANGUAGE C STRICT;

CREATE FUNCTION pg_fs_devname(text)
RETURNS text
AS '$libdir/target_db'
LANGUAGE C STRICT;

CREATE VIEW v_archive_dir AS
SELECT NULL::oid as oid, '<archive_dir>'::text as name,
substring(substring(regexp_split_to_table(setting, E'&&|;'),E'\/[^ ]*\/\%f'),E'\/.*\/') as location
FROM pg_settings
WHERE name='archive_command';

CREATE VIEW v_xlog_dir AS
SELECT NULL::oid as oid, '<pg_xlog>'::text as name , (setting ||'/pg_xlog') as location
FROM pg_settings
WHERE name='data_directory';

CREATE VIEW pg_filesystem AS
SELECT oid, spcname as name,
 CASE
  WHEN spclocation='' THEN
   (SELECT setting FROM pg_settings WHERE name='data_directory')
  ELSE spclocation
 END as location
FROM pg_tablespace
UNION
SELECT oid, name, location
FROM targetdb.v_archive_dir
UNION
SELECT oid, name, location
FROM targetdb.v_xlog_dir;

CREATE VIEW pg_filesystem_size AS
SELECT oid, name, targetdb.pg_fs_devid(location) as device_id, 
       targetdb.pg_fs_devname(location) as device_name, location, 
      (targetdb.pg_fs_size(location) - targetdb.pg_fs_avail(location)) as Used, 
       targetdb.pg_fs_avail(location) as Avail, targetdb.pg_fs_size(location) as Total
FROM targetdb.pg_filesystem;

-- Register functions and views. (check_filling_ratio)
CREATE FUNCTION pg_fillfactor(reloptions text[], relam OID)
RETURNS integer AS
$$
SELECT (regexp_matches(array_to_string($1, '/'),
        'fillfactor=([0-9]+)'))[1]::integer AS fillfactor
UNION ALL
SELECT CASE $2
       WHEN    0 THEN 100 -- heap
       WHEN  403 THEN  90 -- btree
       WHEN  405 THEN  70 -- hash
       WHEN  783 THEN  90 -- gist
       WHEN 2742 THEN 100 -- gin
       END
LIMIT 1;
$$
LANGUAGE sql STABLE;

CREATE VIEW table_sizes AS
SELECT c.oid, nspname AS schemaname, relname, reltuples,
       ceil(reltuples / (8168 * targetdb.pg_fillfactor(reloptions, relam) / 100 /
           (width + 28)))::bigint AS logical_pages,
       (pg_relation_size(c.oid) + CASE reltoastrelid WHEN 0 THEN 0
           ELSE pg_relation_size(reltoastrelid) END) / 8192 AS physical_pages
 FROM pg_class c LEFT JOIN pg_namespace n ON n.oid = c.relnamespace,
      (SELECT starelid, sum(stawidth)::integer + 7 & ~7 AS width
        FROM pg_statistic WHERE staattnum > 0 GROUP BY starelid) AS stat
WHERE c.oid = starelid AND relkind = 'r';

CREATE VIEW check_filling_ratio AS
SELECT oid, schemaname, relname, reltuples, logical_pages, physical_pages,
  CASE physical_pages
      WHEN 0 THEN NULL ELSE logical_pages * 100 / physical_pages END AS tratio
     FROM targetdb.table_sizes
     WHERE schemaname NOT IN ('pg_catalog', 'information_schema')
     ORDER BY tratio DESC;

-- Register functions and views. (check_column_correlation)
CREATE VIEW check_column_correlation AS
SELECT c.oid AS relid, n.nspname AS schemaname, c.relname AS relname, a.attname AS column_name,
       CASE
           WHEN s.stakind1 = 3 THEN s.stanumbers1[1]
           WHEN s.stakind2 = 3 THEN s.stanumbers2[1]
           WHEN s.stakind3 = 3 THEN s.stanumbers3[1]
           WHEN s.stakind4 = 3 THEN s.stanumbers4[1]
           ELSE NULL::real
       END AS correlation
  FROM pg_index i, pg_attribute a, pg_class c,
       pg_namespace n, pg_statistic s
 WHERE indisclustered AND n.oid = c.relnamespace
  AND c.oid = indrelid AND c.oid = attrelid AND c.oid = starelid
  AND attnum = indkey[0] AND attnum = staattnum
  AND has_table_privilege(c.oid, 'select'::text);


-- Register functions and views. (pg_statsinfo)

CREATE OR REPLACE FUNCTION stats_autovacuuminfo(
OUT autovacuum_act_time         timestamp with time zone,
OUT autovacuum_table            text,
OUT autovacuum_num_page_remove  bigint,
OUT autovacuum_num_page_remain  bigint,
OUT autovacuum_num_tup_remove   bigint,
OUT autovacuum_num_tup_remain   bigint,
OUT autovacuum_duration         real
) 
RETURNS setof record
AS '$libdir/target_db','stats_autovacuuminfo'
LANGUAGE C STRICT;

CREATE OR REPLACE FUNCTION stats_checkpointinfo(
OUT checkpoint_act_time       timestamp with time zone,
OUT checkpoint_start_trig     text,
OUT checkpoint_num_buffers    bigint,
OUT checkpoint_create_wal     bigint,
OUT checkpoint_delete_wal     bigint,
OUT checkpoint_recycle_wal    bigint,
OUT checkpoint_write_duration real,
OUT checkpoint_sync_duration  real,
OUT checkpoint_total_duration real
) 
RETURNS setof record
AS '$libdir/target_db','stats_checkpointinfo'
LANGUAGE C STRICT;


CREATE OR REPLACE FUNCTION stats_kernel_info(
OUT kernel_info text
)
RETURNS text
AS '$libdir/target_db','stats_kernel_info'
LANGUAGE C STRICT;


CREATE OR REPLACE FUNCTION stats_cpu_info(
OUT model_name text,
OUT processors int,
OUT cpu_MHz    real,
OUT bogompis   real
)
RETURNS record
AS '$libdir/target_db','stats_cpu_info'
LANGUAGE C STRICT;

CREATE OR REPLACE FUNCTION stats_mem_info(
OUT mem_total  bigint,
OUT mem_free   bigint,
OUT buffers    bigint,
OUT cahced     bigint,
OUT active     bigint,
OUT inactive   bigint,
OUT swap_total bigint,
OUT swap_free  bigint
)
RETURNS record
AS '$libdir/target_db','stats_mem_info'
LANGUAGE C STRICT;


CREATE OR REPLACE FUNCTION stats_hostinfo(
OUT kernel_version text,
OUT processors     int,
OUT mem_total      bigint,
OUT swap_total     bigint) 
RETURNS record
AS $$
  SELECT kernel_info as kernel_version, processors, mem_total, swap_total
  FROM targetdb.stats_kernel_info(), targetdb.stats_cpu_info(), targetdb.stats_mem_info();
$$ LANGUAGE SQL;

CREATE OR REPLACE FUNCTION unlink_stats_autovacuumlogfile(text)
RETURNS boolean
AS $$
SELECT
    pg_file_unlink('pg_statslog/' || $1 || '-autovacuum.log.done')
$$
  LANGUAGE 'sql';

CREATE OR REPLACE FUNCTION unlink_stats_checkpointlogfile()
RETURNS boolean
AS $$
SELECT
    pg_file_unlink('pg_statslog/checkpoint.log.done')
$$
  LANGUAGE 'sql';


-- REVOKE (pg_filesystem)
REVOKE ALL ON FUNCTION pg_fs_size(text) FROM PUBLIC;
REVOKE ALL ON FUNCTION pg_fs_avail(text) FROM PUBLIC;
REVOKE ALL ON FUNCTION pg_fs_devid(text) FROM PUBLIC;
REVOKE ALL ON FUNCTION pg_fs_devname(text) FROM PUBLIC;
REVOKE ALL ON v_xlog_dir FROM PUBLIC;
REVOKE ALL ON v_archive_dir FROM PUBLIC;
REVOKE ALL ON pg_filesystem FROM PUBLIC;
REVOKE ALL ON pg_filesystem_size FROM PUBLIC;

-- REVOKE (check_filling_ratio)
REVOKE ALL ON FUNCTION pg_fillfactor(text[], OID) FROM PUBLIC;
REVOKE ALL ON check_filling_ratio FROM PUBLIC;
REVOKE ALL ON table_sizes FROM PUBLIC;

-- REVOKE (check_column_correlation)
REVOKE ALL ON check_column_correlation FROM PUBLIC;

-- REVOKE (pg_statsinfo)
REVOKE ALL ON FUNCTION stats_autovacuuminfo() FROM PUBLIC;
REVOKE ALL ON FUNCTION stats_checkpointinfo() FROM PUBLIC;
REVOKE ALL ON FUNCTION stats_hostinfo() FROM PUBLIC;
REVOKE ALL ON FUNCTION stats_kernel_info() FROM PUBLIC;
REVOKE ALL ON FUNCTION stats_cpu_info() FROM PUBLIC;
REVOKE ALL ON FUNCTION stats_mem_info() FROM PUBLIC;
REVOKE ALL ON FUNCTION unlink_stats_autovacuumlogfile(text) FROM PUBLIC;
REVOKE ALL ON FUNCTION unlink_stats_checkpointlogfile() FROM PUBLIC;

COMMIT;
